LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
USE ieee.std_logic_signed.ALL;
USE std.textio.ALL;
USE ieee.std_logic_arith.ALL;
USE work.ImPackM.ALL;
USE work.CamModPack.ALL;

-------------------------------------------------------------------------------

ENTITY CamModE IS
  PORT (
    PixCam   : OUT std_logic_vector(7 DOWNTO 0);
    HSyncCam : OUT std_logic;
    VSyncCam : OUT std_logic;
    PClk     : OUT std_logic);
END CamModE;

---------------------------------------------------------------------------------

ARCHITECTURE CamModA OF CamModE IS

  CONSTANT Delta                          : time := 10.416667 ns;  -- 1/8 T von 12 MHz
  SIGNAL   PClk1S, PClk2S, Clk24S, Clk12S : std_logic;

BEGIN

  clocks : PROCESS  -- Erzeugung Taktsignale 12 MHz, 24 MHz, staendiger Pixeltakt
  BEGIN
    PClk1S <= '0';
    Clk24S <= '0';
    Clk12S <= '0';
    WAIT FOR Delta;                     -- Delta
    Clk24S <= '1';
    Clk12S <= '1';
    WAIT FOR Delta;                     -- 2 Delta
    PClk1S <= '1';
    WAIT FOR Delta;                     -- 3 Delta
    Clk24S <= '0';
    WAIT FOR Delta;                     -- 4 Delta
    PClk1S <= '0';
    WAIT FOR Delta;                     -- 5 Delta
    Clk12S <= '0';
    Clk24S <= '1';
    WAIT FOR Delta;                     -- 6 Delta
    PClk1S <= '1';
    WAIT FOR Delta;                     -- 7 Delta
    Clk24S <= '0';
    WAIT FOR Delta;                     -- 8 Delta
  END PROCESS clocks;


  main : PROCESS (Clk12S, PClk1S, Clk24S)
    VARIABLE Row     : it1024  := 1;    -- Zaehlvariable fuer Zeile
    VARIABLE Col     : it1024  := 0;    -- Zaehlvariable fuer Spalte
    VARIABLE Toggle  : bit     := '0';
    VARIABLE RGB565  : std_logic_vector(15 DOWNTO 0);
    FILE FileIn      : text OPEN read_mode IS "bild.txt";  -- nur fuer Variante 2
    VARIABLE LineIn  : line;            -- nur fuer Variante 2
    VARIABLE PixCamV : integer := 0;
  BEGIN  -- process main
-------------------------------------------------------------------------------
-- Erzeugung Zeile und Spalte
    IF Clk12S'event AND Clk12S = '1' THEN
      Col := Col+1;
      IF Col > CHBack+CH2Data+CHFront+CHSync THEN          -- Zeilenende
        Col := 1;
        Row := Row+1;
      END IF;
      IF Row > CVBack + CVFront+CV2Data+CVSync THEN        -- Frameende
        Row := 1;
      END IF;
-- Erzeugung CHSync und CVSync
      IF Col > CHBack AND Col <= CHBack+CH2Data THEN       -- horizontale Sync.
        HSyncCam <= '1';
      ELSE
        HSyncCam <= '0';
      END IF;
      IF Row > CVBack +CV2Data OR Row <= CVBack THEN       -- vertikale Sync.
        VSyncCam <= '0';
      ELSE
        VSyncCam <= '1';
      END IF;
    END IF;
---------------------------------------------------------------------------
-- Erzeugung Pixeltakt mit Luecken
    IF PClk1S'event THEN
      IF ((Row-CVBack)MOD 2) = 0 OR Row > CVBack +CV2Data OR Row <= CVBack THEN
        PClk2S <= '0';
      ELSIF ((Col-CHBack)MOD 2) = 0 OR Col > (CHBack+CH2Data) OR Col <= CHBack THEN
        PClk2S <= '0';
      ELSE
        PClk2S <= PClk1S;
      END IF;
    END IF; 

--======================================================================

-- Erzeugung Kameradaten Variante 1
--
    IF Row > CVBack AND Row <= CVBack+CV2Data THEN
      IF Col > CHBack AND Col < CHBack+CH2Data THEN
        RGB565 := CWeiss;
		  IF Col > CHBack+3*CCol THEN
		  RGB565 := CDunkel;
		  ElSIF Col > CHBack+2*CCol THEN
		  RGB565 := CMittel;
		  ElSIF Col > CHBack+CCol THEN
		  RGB565 := CHell;
		  End IF;
      END IF;



      
      IF Clk24S'event AND Clk24S = '1' THEN
        IF Toggle = '0' THEN
          PixCam <= RGB565(7 DOWNTO 0);
          Toggle := '1';
        ELSE
          PixCam <= RGB565(15 DOWNTO 8);
          Toggle := '0';
        END IF;
      END IF;
    END IF;

--======================================================================
    
 -- Erzeugung Kameradaten Variante 2
--      
--    if Row > CVBack and Row <= CVBack+CV2Data then
--      if Col <= CH2Data+CHBack and Col > CHBack then
--			If Not endfile(FileIn) and rising_edge(Clk24S) then
--				readline(FileIn, LineIn);
--				read(LineIn, PixCamV);
--				
--				PixCam<=conv_std_logic_vector(PixCamV, 8);
--
--			end if;
--
--      end if;
--    end if;
      
--======================================================================
      
  END PROCESS main;

-- Zuweisungen
  PClk <= PClk2S;

END CamModA;
