-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_signed.all;
use work.ImPackM.all;
use work.CamModPack.all;

entity ImInTbE is
end ImInTbE;

architecture ImInTbA of ImInTbE is

	component CamModE 
	  port (
		 PixCam   : out std_logic_vector(7 downto 0);
		 HSyncCam : out std_logic;
		 VSyncCam : out std_logic;
		 PClk     : out std_logic);
	end component;

	component ImWriteE 
	  port (Reset      : in  std_logic;     -- Reset, low-aktiv
			  PixCam   : in  std_logic_vector(7 downto 0);  -- Daten von Kamera
			  HSyncCam : in  std_logic;     -- Hsync von Kamera
			  VSyncCam : in  std_logic;     -- Vsync von Kamera
			  PClk       : in  std_logic;     -- Pixelclock von Kamera
			  ClkW       : out std_logic;     -- ClkW fuer Ram
			  AddrW      : out std_logic_vector(CDepth-1 downto 0);   -- RAM-Adresse a
			  PixW       : out std_logic_vector(CWidth-1 downto 0));  -- Daten Ram
	end component;

  component RamM                        
    port (
      clka  : in  std_logic;
      wea   : in  std_logic_vector(0 downto 0);
      addra : in  std_logic_vector(CDepth-1 downto 0);
      dina  : in  std_logic_vector(CWidth-1 downto 0);
      clkb  : in  std_logic;
      addrb : in  std_logic_vector(CDepth-1 downto 0);
      doutb : out std_logic_vector(CWidth-1 downto 0));
  end component;

  -- Signale CamMod
  signal PixCamS : std_logic_vector(7 downto 0);
  signal HSyncCamS   : std_logic;
  signal VSyncCamS   : std_logic;
  signal PClkS     : std_logic;

   -- Signale RamM
  signal ClkWS  : std_logic;
  signal WeaS   : std_logic_vector(0 downto 0);
  signal AddrWS : std_logic_vector(3 downto 0)  := (others => '0');
  signal PixWS  : std_logic_vector(11 downto 0) := "000100010001";
  signal Clk25S  : std_logic;
  signal AddrRS : std_logic_vector(3 downto 0)  := (others => '0');
  signal PixRS : std_logic_vector(11 downto 0);

  signal ResetS:std_logic;
  
begin

  CamModL : CamModE
    port map (
      PixCam => PixCamS,
      HSyncCam   => HSyncCamS,
      VSyncCam   => VSyncCamS,
      PClk     => PClkS);

  ImWriteL : ImWriteE
    port map (
      Reset    => ResetS,
      PixCam => PixCamS,
      HSyncCam   => HSyncCamS,
      VSyncCam   => VSyncCamS,
      PClk     => PClkS,
      ClkW     => ClkWS,
      AddrW    => AddrWS,
      PixW     => PixWS);

  RamML : RamM
    port map (
      clka  => ClkWS,
      wea   => WeaS,
      addra => AddrWS,
      dina  => PixWS,
      clkb  => Clk25S,
      addrb => AddrRS,
      doutb => PixRS);

-- Reset
  ResetS <= '1';

-- Erzeugung Takt 25
  Takterzeugung_clk25 : process
  begin
    Clk25S <= '0';
    wait for 20 ns;
    Clk25S <= '1';
    wait for 20 ns;
  end process Takterzeugung_clk25;

-- Erzeugung Adresse AddrR
  Adresse_AddrR : process (Clk25S)
  begin  -- process
    if Clk25S'event and Clk25S = '1' then  -- rising clock edge
      if AddrRS = "1011" then
        AddrRS <= "0000";
      else
        AddrRS <= AddrRS + 1;
      end if;
    end if;
  end process Adresse_AddrR;

  WeaS <= "0","1" after 1 us;
  
end ImInTbA;


